import 'package:floative/factory/floative_factory.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  FloativeFactory factory;

  setUpAll(() {
    factory = FloativeFactory.instance;
  });

  test("test the register of singleton in normal conditions", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);
  });

  test("test the register of a LAZY singleton in normal conditions", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst, isLazy: true, overrideInstance: true);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);
  });

  test("try to get a singleton that was not registered", () {
    factory.reset();
    var foo = factory.get<Foo>();

    expect(foo, null);
  });

  test("Register a factory and get it as brand new", () {
    final foo = Foo();

    factory.registerFactory<Foo>(() => foo);

    expect(foo, isInstanceOf<Foo>());
  });

  test("reset the factory", () {
    final foo = Foo();
    final fooNew = FooNew();
    factory.registerFactory<Foo>(() => foo);
    factory.register<FooNew>(() => fooNew);

    factory.reset();

    var fooFromFactory = factory.get<Foo>();
    var fooNewFromFactory = factory.get<FooNew>();
    expect(fooFromFactory, null);
    expect(fooNewFromFactory, null);
  });

  test("test the remove of a factory", () {
    var fooFirst = Foo();

    fooFirst.increment();

    factory.register<Foo>(() => fooFirst);

    var foo = factory.get<Foo>();
    foo.increment();

    expect(foo.value, 2);

    factory.dispose<Foo>();

    var fooAgain = factory.get<Foo>();

    expect(fooAgain, null);
  });
}

class Foo {
  int value = 0;

  void increment() => value++;
}

class FooNew {
  String value = '';

  void giveMeValue() => value = "value of FooNew";
}
